<?php
/**
 * This script receives one GET argument - 'url'. This URL is logged into a links
 * table called ct_Links if it does not already exist and an ID is assigned to it.
 * A 'hit' is then logged against the provided link and the IP and user agent of
 * the user is logged. This script is called from an image tag, so we return
 * a 1x1 transparent GIF at the end.
 *
 * Author: James Caws
 * Website: http://www.jamescaws.co.uk
 * Original post:
 * http://www.jamescaws.co.uk/2008/07/dynamically-count-exit-link-clicks-throughs-using-javascript-php/
 */

/**
* James Caws's script was modified by Stanislaw Polak
* Website: http://www.icsr.agh.edu.pl/~polak/
* Version 1.0 (Dec 9, 2008)
* Modifications:
* 1) Instead of PHP PEAR DB I use PDO (PHP Data Object). More info: http://www.php.net/pdo
* 2) Instead of MySQL I use SQLite
* 3) If tables don't exist, they are created
* 4) Number of clicks is stored in DB
* 5) Logs creation can be switched off
*/
/**
 * Adjustable variables
 */

$LOG=0; 			# 0  means  "Log shouldn't be created", 1 means "Log should be created"
$DBname='/path/DB.sqlite'; 	# the filename of the SQLite database
/** NO NEED TO EDIT BELOW THIS LINE UNLESS YOU WANT TO MAKE CHANGES TO THE STANDARD WAY OF WORKING **/

/**
 * Create DB connection
 */

try{
	$db = new PDO("sqlite:$DBname");
 	$db->setAttribute( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
	/**
 	* Check if 'logs' should be created
 	*/
	if($LOG){
		/**
		* If yes, check if 'ct_Log' table exists
 		*/
		$stmt=$db->query("SELECT COUNT(*) FROM sqlite_master WHERE type='table' AND name='ct_Log'");
		if(! intval($stmt->fetchColumn())){
			/**
 			* if not, create it
 			*/
			$sql= <<<EOD
			CREATE TABLE ct_Log (
			logID INTEGER PRIMARY KEY,
			linkID smallint(6) NOT NULL default '0',
			logTime timestamp NOT NULL default CURRENT_TIMESTAMP,
			ip int(10) NOT NULL default '0',
			useragent  varchar(128) default NULL
			);
EOD;
			$db->query($sql);
		}
	}
	/**
 	* Check if 'ct_Links' table exists
 	*/
	$stmt=$db->query("SELECT COUNT(*) FROM sqlite_master WHERE type='table' AND name='ct_Links'");
	if(! intval($stmt->fetchColumn())){
		/**
 		* if not, create it
 		*/	
		$sql= <<<EOD
		CREATE TABLE ct_Links (
		linkID INTEGER PRIMARY KEY,
		hash varchar(32) UNIQUE NOT NULL default '',
		url varchar(255) NOT NULL default '',
		counter smallint NOT NULL default '1'
		);
EOD;
		$db->query($sql);
	}
}
catch (PDOException $e) {
	print "Error!: " . $e->getMessage() . "<br/>";
  	die();
}

/**
 * Import 'url' param that we want to log a click through against
 */

$sURL = isset($_GET['url']) ? $_GET['url'] : false;

/**
 * Very basic URL validation. This could be improved by only allowing
 * specific scheme types (http, ftp...), verifying host names etc
 */
$aURL = parse_url($sURL);

if (!isset($aURL['scheme']) || !isset($aURL['host'])) {
	$sURL = false;
}

/**
 * If we have a URL, do something with it
 */
if ($sURL) {
	/**
	 * We'll use a hash of the URL for some queries as it means we can set a DB table
	 * with a fixed 32 char column, indexed.
	 */
	$sHash = md5($sURL);
	$sIP = $_SERVER['REMOTE_ADDR'];
	$sUserAgent = $_SERVER['HTTP_USER_AGENT'];

	$iLinkID = local_getLinkID($db, $debugObj, $sHash);

	if ($iLinkID === false) {
		$iLinkID = local_logNewLink($db, $debugObj, $sHash, $sURL);
	}
	else {
		local_incrementLinkCounter($db, $debugObj, $sHash);
	}
	/**
 	* if 'logs' should be created
 	*/
	if($LOG)
		local_logLinkClick($db, $debugObj, $iLinkID, $sIP, $sUserAgent);
}

/**
 * Output transparent 1x1 gif, even though the image will be hidden
 */
header('Content-type: image/gif');
$sTransGIF = 'R0lGODlhAQABAIAAAAAAAAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==';
echo base64_decode($sTransGIF);

/**
 * Here endeth the script
 */
exit;

/**
 * Functions
 */

/**
 * Based on a hash of our loggable URL, get its linkID from our links table, return
 * false if it doesn't exist
 */
function local_getLinkID($db, $debugObj, $sHash) {
	try{
		$stmt=$db->prepare('SELECT linkID FROM ct_Links WHERE hash = ?');
		$stmt->bindParam(1, $sHash);
		$stmt->execute();
		#$debugObj->output("local_getLinkID($db, $debugObj, $sHash) : $sql");
		$iID = $stmt->fetchColumn();
		if (!$iID || $iID == 0) { return false; }
		return $iID;
	}
	catch (PDOException $e) {
		print "Error!: " . $e->getMessage() . "<br/>";
	  	die();
	}
}

/**
 * Based on a hash of our loggable URL, increment a counter
 */
function local_incrementLinkCounter($db, $debugObj, $sHash) {
	try{
		$stmt=$db->prepare('UPDATE ct_Links SET counter = counter +1 WHERE hash = ?');
		$stmt->bindParam(1, $sHash);
		$stmt->execute();
		#$debugObj->output("local_getLinkID($db, $debugObj, $sHash) : $sql");
		return;
	}
	catch (PDOException $e) {
		print "Error!: " . $e->getMessage() . "<br/>";
	  	die();
	}
}

/**
 * Log a new link in the DB and return its linkID
 */
function local_logNewLink($db, $debugObj, $sHash, $sURL) {
	try{
		$stmt=$db->prepare('INSERT INTO ct_Links (hash, url) VALUES (?, ?)');
		$stmt->bindParam(1, $sHash);
		$stmt->bindParam(2, $sURL);
		$stmt->execute();
		#$debugObj->output("local_logNewLink($db, $debugObj, $sHash, $sURL) : $sql");
		return $db->lastInsertId() ;
	}
	catch (PDOException $e) {
		print "Error!: " . $e->getMessage() . "<br/>";
	  	die();
	}
}

/**
 * Prior to calling this function, one or both of local_getLinkID and local_logNewLink
 * should have been called. We'll add an entry to the log table to indicate a URL
 * has been accessed. IP and useragent are logged for information purposes.
 */
function local_logLinkClick($db, $debugObj, $iLinkID, $sIP, $sUserAgent) {
	try{
		$db->sqliteCreateFunction('INET_ATON', 'gethostbyaddr', 1);
		$stmt=$db->prepare('INSERT INTO ct_Log (linkID, ip, useragent) VALUES (?, INET_ATON(?), ?)');
		$stmt->bindParam(1, $iLinkID, PDO::PARAM_INT);
		$stmt->bindParam(2, $sIP);
		$stmt->bindParam(3, $sUserAgent);
		$stmt->execute();
		#$debugObj->output("local_logLinkClick($db, $debugObj, $iLinkID, $sIP, $sUserAgent) : $sql");
		return;
	}
	catch (PDOException $e) {
		print "Error!: " . $e->getMessage() . "<br/>";
	  	die();
	}
}
?>
